"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = __importStar(require("fs"));
var path_1 = __importDefault(require("path"));
var handlebars_1 = __importDefault(require("handlebars"));
var console_1 = require("../services/console");
var dashify_1 = __importDefault(require("dashify"));
var constants_1 = require("../services/constants");
var colors_1 = __importDefault(require("colors"));
function init(program, projectPath, options) {
    var verbosity = program.opts().verbose;
    var logger = new console_1.Logger(verbosity);
    logger.verbose('Run init');
    var augOps = getAugmentedInitOptions(options);
    var files = getFilesToCreate(projectPath, augOps);
    checkFolderEmptyness(program, Object.values(files));
    logger.ok('Creating Mamoru project files');
    createFile(logger, augOps, constants_1.TEMPLATES.PACKAGE_JSON, files.PACKAGE_JSON);
    createFile(logger, augOps, constants_1.TEMPLATES.MANIFEST, files.MANIFEST);
    createFile(logger, augOps, constants_1.TEMPLATES.README, files.README);
    createFile(logger, augOps, constants_1.TEMPLATES.GITIGNORE, files.GITIGNORE);
    if (options.type === 'sql') {
        createFile(logger, augOps, constants_1.TEMPLATES.QUERIES, files.QUERIES);
    }
    if (options.type === 'wasm') {
        createFolder(logger, projectPath, 'src');
        createFolder(logger, projectPath, 'test');
        createFile(logger, augOps, constants_1.TEMPLATES.WASM_INDEX, files.WASM_INDEX);
        createFile(logger, augOps, constants_1.TEMPLATES.WASM_TEST, files.WASM_TEST);
    }
    logger.ok('Mamoru project files created!');
    logger.log("\n    \u2139\uFE0F To start working on your project, run:\n        cd ".concat(projectPath, "\n        npm install\n    \u2139\uFE0F To start the project locally, run:\n        npm run start\n    \u2139\uFE0F To know more about how to create your own daemons, visit:\n        ").concat(colors_1.default.blue('https://docs.mamoru.io'), "\n    "));
}
function checkFolderEmptyness(program, paths) {
    paths.forEach(function (p) {
        if (fs.existsSync(p)) {
            var fileName = path_1.default.basename(p);
            program.error("Directory  already contains  a file named \"".concat(fileName, "\", stopping..."));
        }
    });
}
function getAugmentedInitOptions(options) {
    return __assign(__assign({}, options), { jsonTags: JSON.stringify(options.tags.split(',')), kebabName: (0, dashify_1.default)(options.name) });
}
function getFilesToCreate(projectPath, ops) {
    var files = __assign({}, constants_1.FILES);
    if (ops.type === 'sql') {
        delete files.WASM_INDEX;
        delete files.WASM_TEST;
    }
    if (ops.type === 'wasm') {
        delete files.QUERIES;
    }
    var fileDirs = {};
    Object.entries(files).forEach(function (_a) {
        var key = _a[0], name = _a[1];
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        fileDirs[key] = path_1.default.join(projectPath, name);
    });
    return fileDirs;
}
function createFile(logger, ops, templatePath, targetPath) {
    var targetFileName = path_1.default.basename(targetPath);
    logger.verbose("Creating \"".concat(targetFileName, "\""));
    var templateSrc = fs.readFileSync(templatePath).toString('utf-8');
    var template = handlebars_1.default.compile(templateSrc);
    var result = template(ops);
    fs.writeFileSync(targetPath, result);
}
function createFolder(logger, projectPath, folderPath) {
    var folderName = path_1.default.basename(folderPath);
    logger.verbose("Creating \"".concat(folderName, "\" folder"));
    if (!fs.existsSync(path_1.default.join(projectPath, folderPath))) {
        fs.mkdirSync(path_1.default.join(projectPath, folderName));
    }
}
exports.default = {
    init: init,
};
//# sourceMappingURL=init.js.map